import enum
import io
import json

import boto3
import oss2
from qcloud_cos import CosConfig, CosS3Client
from google.cloud import storage
from google.oauth2 import service_account
from google.oauth2.credentials import Credentials


class ObjectStorageType(enum.Enum):
    OSS = 0
    COS = 1
    S3 = 2
    GCS = 3


class ObjectStorageClient(object):
    def __init__(self, objectStorageType, ak_id, ak_secret, token, endpoint, bucket=None):
        self.type = objectStorageType
        self.ak_id = ak_id
        self.ak_secret = ak_secret
        self.token = token
        self.endpoint = endpoint
        self.bucket = bucket
        self.client = None
        if self.type == ObjectStorageType.OSS:
            auth = oss2.StsAuth(self.ak_id, self.ak_secret, self.token)
            self.client = oss2.Bucket(auth, self.endpoint, self.bucket)
        elif self.type == ObjectStorageType.COS:
            cos_config = CosConfig(Region=self.endpoint, SecretId=self.ak_id, SecretKey=self.ak_secret, Token=self.token)
            self.client = CosS3Client(cos_config)
        elif self.type == ObjectStorageType.S3:
            session = boto3.Session(aws_session_token=self.token,
                                    aws_access_key_id=self.ak_id,
                                    aws_secret_access_key=self.ak_secret,
                                    region_name=self.endpoint)
            self.client = session.resource('s3')
        elif self.type == ObjectStorageType.GCS:
            credentials = None
            if self.ak_id is not None and len(self.ak_id) > 0:
                credentials = service_account.Credentials.from_service_account_info(json.loads(self.ak_id))
            elif self.token is not None and len(self.token) > 0:
                credentials = Credentials(token=self.token)
            self.client = storage.Client(project='clickzetta-connector', credentials=credentials)

    def get_stream(self, bucket_name, object_name):
        stream = None
        if self.type == ObjectStorageType.OSS:
            stream = self.client.get_object(object_name).read()
        elif self.type == ObjectStorageType.COS:
            stream = self.client.get_object(Bucket=bucket_name, Key=object_name)['Body'].get_raw_stream()
        elif self.type == ObjectStorageType.S3:
            stream = self.client.Object(bucket_name, object_name).get()['Body'].read()
        elif self.type == ObjectStorageType.GCS:
            bucket = self.client.bucket(bucket_name)
            blob = bucket.blob(object_name)
            file_obj = io.BytesIO()
            blob.download_to_file(file_obj)
            file_obj.seek(0)
            stream = file_obj
        return stream







# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: compression.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11\x63ompression.proto\x12\x04kudu*w\n\x0f\x43ompressionType\x12\x18\n\x13UNKNOWN_COMPRESSION\x10\xe7\x07\x12\x17\n\x13\x44\x45\x46\x41ULT_COMPRESSION\x10\x00\x12\x12\n\x0eNO_COMPRESSION\x10\x01\x12\n\n\x06SNAPPY\x10\x02\x12\x07\n\x03LZ4\x10\x03\x12\x08\n\x04ZLIB\x10\x04\x42\x11\n\x0forg.apache.kudu')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'compression_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\017org.apache.kudu'
  _globals['_COMPRESSIONTYPE']._serialized_start=27
  _globals['_COMPRESSIONTYPE']._serialized_end=146
# @@protoc_insertion_point(module_scope)



import typing

def split_sql(query: str) -> typing.List[str]:
    ret = []
    c = None # current char
    p = None # previous char
    b = 0 # begin of current sql
    state = 1 # current state
    NORMAL = 1
    IDENTIFIER = 2
    SINGLE_QUOTATION = 3
    DOUBLE_QUOTATION = 4
    SINGLE_LINE_COMMENT = 5
    MULTI_LINE_COMMENT = 6
    for i in range(0, len(query)):
        c = query[i]
        if state == NORMAL:
            if c == ';':
                if i - b > 0:
                    ret.append(query[b:i])
                b = i + 1
                p = None
            elif p == '-' and c == '-':
                state = SINGLE_LINE_COMMENT
                p = None
            elif p == '/' and c == '*':
                state = MULTI_LINE_COMMENT
                p = None
            elif c == '`':
                state = IDENTIFIER
                p = None
            elif c == "'":
                state = SINGLE_QUOTATION
                p = None
            elif c == '"':
                state = DOUBLE_QUOTATION
                p = None
            else:
                p = c
        elif state == IDENTIFIER:
            if c == '`' and p != '\\':
                state = NORMAL
                p = None
            else:
                p = c
        elif state == SINGLE_QUOTATION:
            if c == "'" and p != '\\':
                state = NORMAL
                p = None
            elif p == '\\':
                p = None
            else:
                p = c
        elif state == DOUBLE_QUOTATION:
            if c == '"' and p != '\\':
                state = NORMAL
                p = None
            elif p == '\\':
                p = None
            else:
                p = c
        elif state == SINGLE_LINE_COMMENT:
            if c == '\n':
                state = NORMAL
                p = None
            else:
                p = c
        elif state == MULTI_LINE_COMMENT:
            if p == '*' and c == '/':
                state = NORMAL
                p = None
            else:
                p = c

    if b < len(query):
        ret.append(query[b:])
    return ret

# def myassert(sql, num, sqls):
#     if len(sqls) == num:
#         print('-- OK --')
#     else:
#         print('-- FAIL --')
#     print(sql)
#     for s in sqls:
#         print(f'[{s}]')

# if __name__ == "__main__":
#     sql = "select 1";
#     sqls = split_sql(sql);
#     myassert(sql, 1, sqls);

#     sql = "select 1;";
#     sqls = split_sql(sql);
#     myassert(sql, 1, sqls);

#     sql = "select 1;select 2";
#     sqls = split_sql(sql);
#     myassert(sql, 2, sqls);

#     sql = "select 1;select 2;";
#     sqls = split_sql(sql);
#     myassert(sql, 2, sqls);

#     sql = "select 1\n\n\nfrom table;";
#     sqls = split_sql(sql);
#     myassert(sql, 1, sqls);

#     sql = ";";
#     sqls = split_sql(sql);
#     myassert(sql, 0, sqls);

#     sql = ";;";
#     sqls = split_sql(sql);
#     myassert(sql, 0, sqls);

#     sql = "; ;";
#     sqls = split_sql(sql);
#     myassert(sql, 1, sqls);

#     sql = ";\n;";
#     sqls = split_sql(sql);
#     myassert(sql, 1, sqls);

#     sql = "";
#     sqls = split_sql(sql);
#     myassert(sql, 0, sqls);

#     sql = "\n";
#     sqls = split_sql(sql);
#     myassert(sql, 1, sqls);

#     sql = "select *\n-- -- ;\nfrom world\n";
#     sqls = split_sql(sql);
#     myassert(sql, 1, sqls);

#     sql = "select *\n-- -- ;\nfrom world\n";
#     sqls = split_sql(sql);
#     myassert(sql, 1, sqls);

#     sql = "select `aaaa";
#     sqls = split_sql(sql);
#     myassert(sql, 1, sqls);

#     sql = "select 'aaa;\nbbb'\n";
#     sqls = split_sql(sql);
#     myassert(sql, 1, sqls);

#     sql = "select \"--\\\"/*;\n*/\"";
#     sqls = split_sql(sql);
#     myassert(sql, 1, sqls);

#     sql = "-- line 1\nselect\n/* comment -- -- ;\n****/*\nfrom foo";
#     sqls = split_sql(sql);
#     myassert(sql, 1, sqls);

#     sql = "/*/--/*/;\nselect /* -- 1; */\n1;-- sql 2";
#     sqls = split_sql(sql);
#     myassert(sql, 3, sqls);

#     sql = """select "1\\\\";select2
# """
#     sqls = split_sql(sql);
#     myassert(sql, 2, sqls);



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: kudu_common.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import row_operations_pb2 as row__operations__pb2
from . import block_bloom_filter_pb2 as block__bloom__filter__pb2
from . import compression_pb2 as compression__pb2
from . import hash_pb2 as hash__pb2
from . import pb_util_pb2 as pb__util__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11kudu_common.proto\x12\x04kudu\x1a\x14row_operations.proto\x1a\x18\x62lock_bloom_filter.proto\x1a\x11\x63ompression.proto\x1a\nhash.proto\x1a\rpb_util.proto\"b\n\x16\x43olumnTypeAttributesPB\x12\x11\n\tprecision\x18\x01 \x01(\x05\x12\r\n\x05scale\x18\x02 \x01(\x05\x12\x0e\n\x06length\x18\x03 \x01(\x05\x12\x16\n\x0especialColCode\x18\x04 \x01(\x05\"\x8f\x03\n\x0e\x43olumnSchemaPB\x12\n\n\x02id\x18\x01 \x01(\r\x12\x0c\n\x04name\x18\x02 \x02(\t\x12\x1c\n\x04type\x18\x03 \x02(\x0e\x32\x0e.kudu.DataType\x12\x15\n\x06is_key\x18\x04 \x01(\x08:\x05\x66\x61lse\x12\x1a\n\x0bis_nullable\x18\x05 \x01(\x08:\x05\x66\x61lse\x12\x1a\n\x12read_default_value\x18\x06 \x01(\x0c\x12\x1b\n\x13write_default_value\x18\x07 \x01(\x0c\x12\x33\n\x08\x65ncoding\x18\x08 \x01(\x0e\x32\x12.kudu.EncodingType:\rAUTO_ENCODING\x12?\n\x0b\x63ompression\x18\t \x01(\x0e\x32\x15.kudu.CompressionType:\x13\x44\x45\x46\x41ULT_COMPRESSION\x12\x1b\n\x10\x63\x66ile_block_size\x18\n \x01(\x05:\x01\x30\x12\x35\n\x0ftype_attributes\x18\x0b \x01(\x0b\x32\x1c.kudu.ColumnTypeAttributesPB\x12\x0f\n\x07\x63omment\x18\x0c \x01(\t\"\xdf\x01\n\x13\x43olumnSchemaDeltaPB\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08new_name\x18\x02 \x01(\t\x12\x15\n\rdefault_value\x18\x04 \x01(\x0c\x12\x16\n\x0eremove_default\x18\x05 \x01(\x08\x12$\n\x08\x65ncoding\x18\x06 \x01(\x0e\x32\x12.kudu.EncodingType\x12*\n\x0b\x63ompression\x18\x07 \x01(\x0e\x32\x15.kudu.CompressionType\x12\x12\n\nblock_size\x18\x08 \x01(\x05\x12\x13\n\x0bnew_comment\x18\t \x01(\t\"1\n\x08SchemaPB\x12%\n\x07\x63olumns\x18\x01 \x03(\x0b\x32\x14.kudu.ColumnSchemaPB\"(\n\nHostPortPB\x12\x0c\n\x04host\x18\x01 \x02(\t\x12\x0c\n\x04port\x18\x02 \x02(\r\"\xab\x05\n\x11PartitionSchemaPB\x12?\n\x0bhash_schema\x18\x01 \x03(\x0b\x32*.kudu.PartitionSchemaPB.HashBucketSchemaPB\x12;\n\x0crange_schema\x18\x02 \x01(\x0b\x32%.kudu.PartitionSchemaPB.RangeSchemaPB\x12P\n\x19\x63ustom_hash_schema_ranges\x18\x05 \x03(\x0b\x32-.kudu.PartitionSchemaPB.RangeWithHashSchemaPB\x1a@\n\x12\x43olumnIdentifierPB\x12\x0c\n\x02id\x18\x01 \x01(\x05H\x00\x12\x0e\n\x04name\x18\x02 \x01(\tH\x00\x42\x0c\n\nidentifier\x1aL\n\rRangeSchemaPB\x12;\n\x07\x63olumns\x18\x01 \x03(\x0b\x32*.kudu.PartitionSchemaPB.ColumnIdentifierPB\x1a\xa1\x01\n\x12HashBucketSchemaPB\x12;\n\x07\x63olumns\x18\x01 \x03(\x0b\x32*.kudu.PartitionSchemaPB.ColumnIdentifierPB\x12\x13\n\x0bnum_buckets\x18\x02 \x02(\x05\x12\x0c\n\x04seed\x18\x03 \x01(\r\x12+\n\x0ehash_algorithm\x18\x04 \x01(\x0e\x32\x13.kudu.HashAlgorithm\x1a\x85\x01\n\x15RangeWithHashSchemaPB\x12+\n\x0crange_bounds\x18\x01 \x01(\x0b\x32\x15.kudu.RowOperationsPB\x12?\n\x0bhash_schema\x18\x02 \x03(\x0b\x32*.kudu.PartitionSchemaPB.HashBucketSchemaPBJ\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05\"_\n\x0bPartitionPB\x12\x18\n\x0chash_buckets\x18\x01 \x03(\x05\x42\x02\x10\x01\x12\x1b\n\x13partition_key_start\x18\x02 \x01(\x0c\x12\x19\n\x11partition_key_end\x18\x03 \x01(\x0c\"\xef\x04\n\x11\x43olumnPredicatePB\x12\x0e\n\x06\x63olumn\x18\x01 \x01(\t\x12.\n\x05range\x18\x02 \x01(\x0b\x32\x1d.kudu.ColumnPredicatePB.RangeH\x00\x12\x34\n\x08\x65quality\x18\x03 \x01(\x0b\x32 .kudu.ColumnPredicatePB.EqualityH\x00\x12\x38\n\x0bis_not_null\x18\x04 \x01(\x0b\x32!.kudu.ColumnPredicatePB.IsNotNullH\x00\x12\x31\n\x07in_list\x18\x05 \x01(\x0b\x32\x1e.kudu.ColumnPredicatePB.InListH\x00\x12\x31\n\x07is_null\x18\x06 \x01(\x0b\x32\x1e.kudu.ColumnPredicatePB.IsNullH\x00\x12@\n\x0fin_bloom_filter\x18\x07 \x01(\x0b\x32%.kudu.ColumnPredicatePB.InBloomFilterH\x00\x1a\x31\n\x05Range\x12\x13\n\x05lower\x18\x01 \x01(\x0c\x42\x04\x88\xb5\x18\x01\x12\x13\n\x05upper\x18\x02 \x01(\x0c\x42\x04\x88\xb5\x18\x01\x1a\x1f\n\x08\x45quality\x12\x13\n\x05value\x18\x01 \x01(\x0c\x42\x04\x88\xb5\x18\x01\x1a\x1e\n\x06InList\x12\x14\n\x06values\x18\x01 \x03(\x0c\x42\x04\x88\xb5\x18\x01\x1a\x0b\n\tIsNotNull\x1a\x08\n\x06IsNull\x1aj\n\rInBloomFilter\x12/\n\rbloom_filters\x18\x01 \x03(\x0b\x32\x18.kudu.BlockBloomFilterPB\x12\x13\n\x05lower\x18\x02 \x01(\x0c\x42\x04\x88\xb5\x18\x01\x12\x13\n\x05upper\x18\x03 \x01(\x0c\x42\x04\x88\xb5\x18\x01\x42\x0b\n\tpredicate\"k\n\nKeyRangePB\x12\x1f\n\x11start_primary_key\x18\x01 \x01(\x0c\x42\x04\x88\xb5\x18\x01\x12\x1e\n\x10stop_primary_key\x18\x02 \x01(\x0c\x42\x04\x88\xb5\x18\x01\x12\x1c\n\x14size_bytes_estimates\x18\x03 \x02(\x04\"k\n\x12TableExtraConfigPB\x12\x1b\n\x13history_max_age_sec\x18\x01 \x01(\x05\x12\x1c\n\x14maintenance_priority\x18\x02 \x01(\x05\x12\x1a\n\x12\x64isable_compaction\x18\x03 \x01(\x08*\xa6\x02\n\x08\x44\x61taType\x12\x11\n\x0cUNKNOWN_DATA\x10\xe7\x07\x12\t\n\x05UINT8\x10\x00\x12\x08\n\x04INT8\x10\x01\x12\n\n\x06UINT16\x10\x02\x12\t\n\x05INT16\x10\x03\x12\n\n\x06UINT32\x10\x04\x12\t\n\x05INT32\x10\x05\x12\n\n\x06UINT64\x10\x06\x12\t\n\x05INT64\x10\x07\x12\n\n\x06STRING\x10\x08\x12\x08\n\x04\x42OOL\x10\t\x12\t\n\x05\x46LOAT\x10\n\x12\n\n\x06\x44OUBLE\x10\x0b\x12\n\n\x06\x42INARY\x10\x0c\x12\x13\n\x0fUNIXTIME_MICROS\x10\r\x12\n\n\x06INT128\x10\x0e\x12\r\n\tDECIMAL32\x10\x0f\x12\r\n\tDECIMAL64\x10\x10\x12\x0e\n\nDECIMAL128\x10\x11\x12\x0e\n\nIS_DELETED\x10\x12\x12\x0b\n\x07VARCHAR\x10\x13\x12\x08\n\x04\x44\x41TE\x10\x14*\xa0\x01\n\x0c\x45ncodingType\x12\x15\n\x10UNKNOWN_ENCODING\x10\xe7\x07\x12\x11\n\rAUTO_ENCODING\x10\x00\x12\x12\n\x0ePLAIN_ENCODING\x10\x01\x12\x13\n\x0fPREFIX_ENCODING\x10\x02\x12\x10\n\x0cGROUP_VARINT\x10\x03\x12\x07\n\x03RLE\x10\x04\x12\x11\n\rDICT_ENCODING\x10\x05\x12\x0f\n\x0b\x42IT_SHUFFLE\x10\x06*l\n\x07HmsMode\x12\x08\n\x04NONE\x10\x00\x12\x1a\n\x16\x44ISABLE_HIVE_METASTORE\x10\x03\x12\x19\n\x15\x45NABLE_HIVE_METASTORE\x10\x01\x12 \n\x1c\x45NABLE_METASTORE_INTEGRATION\x10\x02*h\n\x17\x45xternalConsistencyMode\x12%\n!UNKNOWN_EXTERNAL_CONSISTENCY_MODE\x10\x00\x12\x15\n\x11\x43LIENT_PROPAGATED\x10\x01\x12\x0f\n\x0b\x43OMMIT_WAIT\x10\x02*^\n\x08ReadMode\x12\x15\n\x11UNKNOWN_READ_MODE\x10\x00\x12\x0f\n\x0bREAD_LATEST\x10\x01\x12\x14\n\x10READ_AT_SNAPSHOT\x10\x02\x12\x14\n\x10READ_YOUR_WRITES\x10\x03*?\n\tOrderMode\x12\x16\n\x12UNKNOWN_ORDER_MODE\x10\x00\x12\r\n\tUNORDERED\x10\x01\x12\x0b\n\x07ORDERED\x10\x02*W\n\x10ReplicaSelection\x12\x1d\n\x19UNKNOWN_REPLICA_SELECTION\x10\x00\x12\x0f\n\x0bLEADER_ONLY\x10\x01\x12\x13\n\x0f\x43LOSEST_REPLICA\x10\x02*6\n\x0bTableTypePB\x12\x11\n\rDEFAULT_TABLE\x10\x00\x12\x14\n\x10TXN_STATUS_TABLE\x10\x01\x42\x11\n\x0forg.apache.kudu')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'kudu_common_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\017org.apache.kudu'
  _globals['_PARTITIONPB'].fields_by_name['hash_buckets']._loaded_options = None
  _globals['_PARTITIONPB'].fields_by_name['hash_buckets']._serialized_options = b'\020\001'
  _globals['_COLUMNPREDICATEPB_RANGE'].fields_by_name['lower']._loaded_options = None
  _globals['_COLUMNPREDICATEPB_RANGE'].fields_by_name['lower']._serialized_options = b'\210\265\030\001'
  _globals['_COLUMNPREDICATEPB_RANGE'].fields_by_name['upper']._loaded_options = None
  _globals['_COLUMNPREDICATEPB_RANGE'].fields_by_name['upper']._serialized_options = b'\210\265\030\001'
  _globals['_COLUMNPREDICATEPB_EQUALITY'].fields_by_name['value']._loaded_options = None
  _globals['_COLUMNPREDICATEPB_EQUALITY'].fields_by_name['value']._serialized_options = b'\210\265\030\001'
  _globals['_COLUMNPREDICATEPB_INLIST'].fields_by_name['values']._loaded_options = None
  _globals['_COLUMNPREDICATEPB_INLIST'].fields_by_name['values']._serialized_options = b'\210\265\030\001'
  _globals['_COLUMNPREDICATEPB_INBLOOMFILTER'].fields_by_name['lower']._loaded_options = None
  _globals['_COLUMNPREDICATEPB_INBLOOMFILTER'].fields_by_name['lower']._serialized_options = b'\210\265\030\001'
  _globals['_COLUMNPREDICATEPB_INBLOOMFILTER'].fields_by_name['upper']._loaded_options = None
  _globals['_COLUMNPREDICATEPB_INBLOOMFILTER'].fields_by_name['upper']._serialized_options = b'\210\265\030\001'
  _globals['_KEYRANGEPB'].fields_by_name['start_primary_key']._loaded_options = None
  _globals['_KEYRANGEPB'].fields_by_name['start_primary_key']._serialized_options = b'\210\265\030\001'
  _globals['_KEYRANGEPB'].fields_by_name['stop_primary_key']._loaded_options = None
  _globals['_KEYRANGEPB'].fields_by_name['stop_primary_key']._serialized_options = b'\210\265\030\001'
  _globals['_DATATYPE']._serialized_start=2570
  _globals['_DATATYPE']._serialized_end=2864
  _globals['_ENCODINGTYPE']._serialized_start=2867
  _globals['_ENCODINGTYPE']._serialized_end=3027
  _globals['_HMSMODE']._serialized_start=3029
  _globals['_HMSMODE']._serialized_end=3137
  _globals['_EXTERNALCONSISTENCYMODE']._serialized_start=3139
  _globals['_EXTERNALCONSISTENCYMODE']._serialized_end=3243
  _globals['_READMODE']._serialized_start=3245
  _globals['_READMODE']._serialized_end=3339
  _globals['_ORDERMODE']._serialized_start=3341
  _globals['_ORDERMODE']._serialized_end=3404
  _globals['_REPLICASELECTION']._serialized_start=3406
  _globals['_REPLICASELECTION']._serialized_end=3493
  _globals['_TABLETYPEPB']._serialized_start=3495
  _globals['_TABLETYPEPB']._serialized_end=3549
  _globals['_COLUMNTYPEATTRIBUTESPB']._serialized_start=121
  _globals['_COLUMNTYPEATTRIBUTESPB']._serialized_end=219
  _globals['_COLUMNSCHEMAPB']._serialized_start=222
  _globals['_COLUMNSCHEMAPB']._serialized_end=621
  _globals['_COLUMNSCHEMADELTAPB']._serialized_start=624
  _globals['_COLUMNSCHEMADELTAPB']._serialized_end=847
  _globals['_SCHEMAPB']._serialized_start=849
  _globals['_SCHEMAPB']._serialized_end=898
  _globals['_HOSTPORTPB']._serialized_start=900
  _globals['_HOSTPORTPB']._serialized_end=940
  _globals['_PARTITIONSCHEMAPB']._serialized_start=943
  _globals['_PARTITIONSCHEMAPB']._serialized_end=1626
  _globals['_PARTITIONSCHEMAPB_COLUMNIDENTIFIERPB']._serialized_start=1172
  _globals['_PARTITIONSCHEMAPB_COLUMNIDENTIFIERPB']._serialized_end=1236
  _globals['_PARTITIONSCHEMAPB_RANGESCHEMAPB']._serialized_start=1238
  _globals['_PARTITIONSCHEMAPB_RANGESCHEMAPB']._serialized_end=1314
  _globals['_PARTITIONSCHEMAPB_HASHBUCKETSCHEMAPB']._serialized_start=1317
  _globals['_PARTITIONSCHEMAPB_HASHBUCKETSCHEMAPB']._serialized_end=1478
  _globals['_PARTITIONSCHEMAPB_RANGEWITHHASHSCHEMAPB']._serialized_start=1481
  _globals['_PARTITIONSCHEMAPB_RANGEWITHHASHSCHEMAPB']._serialized_end=1614
  _globals['_PARTITIONPB']._serialized_start=1628
  _globals['_PARTITIONPB']._serialized_end=1723
  _globals['_COLUMNPREDICATEPB']._serialized_start=1726
  _globals['_COLUMNPREDICATEPB']._serialized_end=2349
  _globals['_COLUMNPREDICATEPB_RANGE']._serialized_start=2091
  _globals['_COLUMNPREDICATEPB_RANGE']._serialized_end=2140
  _globals['_COLUMNPREDICATEPB_EQUALITY']._serialized_start=2142
  _globals['_COLUMNPREDICATEPB_EQUALITY']._serialized_end=2173
  _globals['_COLUMNPREDICATEPB_INLIST']._serialized_start=2175
  _globals['_COLUMNPREDICATEPB_INLIST']._serialized_end=2205
  _globals['_COLUMNPREDICATEPB_ISNOTNULL']._serialized_start=2207
  _globals['_COLUMNPREDICATEPB_ISNOTNULL']._serialized_end=2218
  _globals['_COLUMNPREDICATEPB_ISNULL']._serialized_start=2220
  _globals['_COLUMNPREDICATEPB_ISNULL']._serialized_end=2228
  _globals['_COLUMNPREDICATEPB_INBLOOMFILTER']._serialized_start=2230
  _globals['_COLUMNPREDICATEPB_INBLOOMFILTER']._serialized_end=2336
  _globals['_KEYRANGEPB']._serialized_start=2351
  _globals['_KEYRANGEPB']._serialized_end=2458
  _globals['_TABLEEXTRACONFIGPB']._serialized_start=2460
  _globals['_TABLEEXTRACONFIGPB']._serialized_end=2567
# @@protoc_insertion_point(module_scope)



"""Schemas for ClickZetta tables / queries."""

import collections
import enum
from typing import Any, Dict, Iterable, Union

import standard_sql
from .enums import StandardSqlTypeNames

_STRUCT_TYPES = ("MAP", "STRUCT", "ARRAY")

LEGACY_TO_STANDARD_TYPES = {
    "STRING": StandardSqlTypeNames.STRING,
    "BINARY": StandardSqlTypeNames.BINARY,
    "INT8": StandardSqlTypeNames.INT8,
    "INT16": StandardSqlTypeNames.INT16,
    "FLOAT32": StandardSqlTypeNames.FLOAT32,
    "FLOAT64": StandardSqlTypeNames.FLOAT64,
    "INT32": StandardSqlTypeNames.INT32,
    "INT64": StandardSqlTypeNames.INT64,
    "BOOL": StandardSqlTypeNames.BOOL,
    "MAP": StandardSqlTypeNames.MAP,
    "ARRAY": StandardSqlTypeNames.ARRAY,
    "STRUCT": StandardSqlTypeNames.STRUCT,
    "TIMESTAMP": StandardSqlTypeNames.TIMESTAMP,
    "DATE": StandardSqlTypeNames.DATE,
    "DECIMAL": StandardSqlTypeNames.DECIMAL,
    "VARCHAR": StandardSqlTypeNames.VARCHAR,
    "CHAR": StandardSqlTypeNames.CHAR,
}


class _DefaultSentinel(enum.Enum):
    DEFAULT_VALUE = object()


_DEFAULT_VALUE = _DefaultSentinel.DEFAULT_VALUE


class SchemaField(object):

    def __init__(
            self,
            name: str,
            field_type: str,
            mode: str = "NULLABLE",
            description: Union[str, _DefaultSentinel] = _DEFAULT_VALUE,
            fields: Iterable["SchemaField"] = (),
            precision: Union[int, _DefaultSentinel] = _DEFAULT_VALUE,
            scale: Union[int, _DefaultSentinel] = _DEFAULT_VALUE,
            max_length: Union[int, _DefaultSentinel] = _DEFAULT_VALUE,
    ):
        self._properties: Dict[str, Any] = {
            "name": name,
            "type": field_type,
        }
        if mode is not None:
            self._properties["mode"] = mode.upper()
        if description is not _DEFAULT_VALUE:
            self._properties["description"] = description
        if precision is not _DEFAULT_VALUE:
            self._properties["precision"] = precision
        if scale is not _DEFAULT_VALUE:
            self._properties["scale"] = scale
        if max_length is not _DEFAULT_VALUE:
            self._properties["maxLength"] = max_length
        self._fields = tuple(fields)

    @staticmethod
    def __get_int(api_repr, name):
        v = api_repr.get(name, _DEFAULT_VALUE)
        if v is not _DEFAULT_VALUE:
            v = int(v)
        return v

    @classmethod
    def from_api_repr(cls, api_repr: dict) -> "SchemaField":
        field_type = api_repr["type"].upper()
        mode = api_repr.get("mode", "NULLABLE")
        description = api_repr.get("description", _DEFAULT_VALUE)
        fields = api_repr.get("fields", ())

        return cls(
            field_type=field_type,
            fields=[cls.from_api_repr(f) for f in fields],
            mode=mode.upper(),
            description=description,
            name=api_repr["name"],
            precision=cls.__get_int(api_repr, "precision"),
            scale=cls.__get_int(api_repr, "scale"),
            max_length=cls.__get_int(api_repr, "maxLength"),
        )

    @property
    def name(self):
        return self._properties["name"]

    @property
    def field_type(self):
        return self._properties["type"]

    @property
    def mode(self):
        return self._properties.get("mode")

    @property
    def is_nullable(self):
        return self.mode == "NULLABLE"

    @property
    def description(self):
        return self._properties.get("description")

    @property
    def precision(self):
        return self._properties.get("precision")

    @property
    def scale(self):
        return self._properties.get("scale")

    @property
    def max_length(self):
        return self._properties.get("maxLength")

    @property
    def fields(self):
        return self._fields

    def to_api_repr(self) -> dict:
        answer = self._properties.copy()
        if self.field_type.upper() in _STRUCT_TYPES:
            answer["fields"] = [f.to_api_repr() for f in self.fields]

        return answer

    def _key(self):
        field_type = self.field_type.upper() if self.field_type is not None else None
        if field_type is not None:
            if field_type == "STRING" or field_type == "VARCHAR" or field_type == "CHAR":
                if self.max_length is not None:
                    field_type = f"{field_type}({self.max_length})"
            elif field_type.startswith("DECIMAL"):
                if self.precision is not None:
                    if self.scale is not None:
                        field_type = f"{field_type}({self.precision}, {self.scale})"
                    else:
                        field_type = f"{field_type}({self.precision})"

        policy_tags = (
            None if self.policy_tags is None else tuple(sorted(self.policy_tags.names))
        )

        return (
            self.name,
            field_type,
            self.mode.upper(),
            self.description,
            self._fields,
        )

    def to_standard_sql(self) -> standard_sql.StandardSqlField:
        sql_type = standard_sql.StandardSqlDataType()

        sql_type.type_kind = LEGACY_TO_STANDARD_TYPES.get(
            self.field_type,
            StandardSqlTypeNames.TYPE_KIND_UNSPECIFIED,
        )

        return standard_sql.StandardSqlField(name=self.name, type=sql_type)

    def __eq__(self, other):
        if not isinstance(other, SchemaField):
            return NotImplemented
        return self._key() == other._key()

    def __ne__(self, other):
        return not self == other

    def __hash__(self):
        return hash(self._key())

    def __repr__(self):
        key = self._key()
        adjusted_key = key[:-1]
        return f"{self.__class__.__name__}{adjusted_key}"


def _parse_schema_resource(info):
    return [SchemaField.from_api_repr(f) for f in info.get("fields", ())]


def _build_schema_resource(fields):
    return [field.to_api_repr() for field in fields]


def _to_schema_fields(schema):
    for field in schema:
        if not isinstance(field, (SchemaField, collections.abc.Mapping)):
            raise ValueError(
                "Schema items must either be fields or compatible "
                "mapping representations."
            )

    return [
        field if isinstance(field, SchemaField) else SchemaField.from_api_repr(field)
        for field in schema
    ]


import os
import platform
import subprocess


def build_proto():
    PROJECT_SRC_DIR = os.path.abspath(os.path.join(os.getcwd(), "../../../.."))
    CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))
    PROTO_DIR = os.path.abspath(os.path.join(CURRENT_DIR, "proto/source"))
    PROTO_OUT_DIR = os.path.abspath(os.path.join(CURRENT_DIR, "proto/generated"))

    os.makedirs(PROTO_OUT_DIR, exist_ok=True)

    for source_file in os.listdir(PROTO_DIR):
        subprocess.call(
            'python3 -m grpc_tools.protoc -I . --python_out=' + PROTO_OUT_DIR + ' --grpc_python_out=' + PROTO_OUT_DIR
            + ' --proto_path =' + PROTO_DIR + ' '
            + os.path.abspath(os.path.join(PROTO_DIR, source_file)), shell=True)

    for generated_file in os.listdir(PROTO_OUT_DIR):
        if platform.system() == "Darwin":
            subprocess.call("sed -i '' 's/^import /from . import /' " + os.path.abspath(
                os.path.join(PROTO_OUT_DIR, generated_file)), shell=True)
        elif platform.system() == "Linux":
            subprocess.call("sed -i 's/^import /from . import /' " + os.path.abspath(
                os.path.join(PROTO_OUT_DIR, generated_file)), shell=True)